/* 
 * ncovert2.c - main prog file, includes help files and processes args
 *
 * $Id: ncovert2.c,v 1.1.1.1 2004/03/17 16:55:29 s-nomad Exp $
 *
 * $Log: ncovert2.c,v $
 * Revision 1.1.1.1  2004/03/17 16:55:29  s-nomad
 * Initial import for ncovert2-1.1
 *
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include "ncovert.h"
#include "config.h"

void print_ver(void)
{
	printf("%s %s  - NMRC Covert Channel\n",PACKAGE,VERSION);
	printf("Simple Nomad - <thegnome@nmrc.org>\n");
	printf("Based in part on Ncovert and Chris Rouland's covert_tcp.\n");
}

void usage(char *p)
{
	printf("Usage:");
	printf("%s -s source_host -d dest_host -f file [options]\n\n", p);
	printf("Options include:\n");
	printf("  -q		quiet mode\n");
	printf("  -r            random source addresses after first packet\n");
	printf("  -S            server mode\n");
	printf("  -t <timeout>  timeout in seconds, server mode only (default 10)\n");
	printf("  -v		verbose output\n");
	printf("  -V		print version info and exit\n");
	printf("  -h            this help screen\n\n");
}

int main(int argc, char **argv)
{
	unsigned int source_host=0,dest_host=0;
	int server=0,file=0,ver=0,quiet=0;
	char ch;
	char *prog;
	char desthost[80],srchost[80],filename[80];
	
	prog = argv[0];
	timeout = 10;
	verbose = 0;
	rand_saddr = 0;
	
	while ((ch = getopt(argc, argv, "?hf:s:d:t:SVvqr")) != EOF)
		switch(ch)
		{
			case '?':
			case 'h':
				usage(prog);
				exit(0);
			case 'f':
				strncpy(filename,optarg,79);
				file=1;
				break;
			case 's':
				source_host=host_convert(optarg);
				strncpy(srchost,optarg,79);
				break;
			case 'd':
				dest_host=host_convert(optarg);
				strncpy(desthost,optarg,79);
				break;
			case 't':
				timeout=atoi(optarg);
				break;
			case 'S':
				server=1;
				break;
			case 'V':
				ver=1;
				break;
			case 'v':
				verbose++;
				break;
			case 'q':
				quiet++;
				break;
			case 'r':
				rand_saddr=1;
				break;
			default:
				usage(prog);
				exit(-1);
		}

	if(ver)
	{
		print_ver();
		exit(0);
	}

	

	if(verbose && quiet)
	{
		printf("\n*** You can't choose both verbose and quiet modes\n\n");
		usage(prog);
		exit(-1);
	}

	if(!quiet) print_ver();
	
	if(geteuid() !=0)
	{
		printf("\n*** You need to be root to run this.\n\n");
		exit(0);
	}

	if(mlockall(MCL_CURRENT|MCL_FUTURE) == -1)
	{
		fprintf(stderr,"\n*** Unable to lock memory from paging to swap\n");
		exit(-1);
	}
	
	if(file != 1)
	{
		printf("\n*** You must supply a filename\n\n");
		usage(prog);
		exit(-1);
	}
	if(source_host == 0)
	{
		printf("\n*** You must supply a source host\n\n");
		usage(prog);
		exit(-1);
	}
	if(dest_host == 0)
	{
		printf("\n*** You must supply a destination host\n\n");
		usage(prog);
		exit(-1);
	}
	
	if(server==0) /* if they want to be a client do this... */
	{   
		if (verbose)
		{
			printf("Sending data to  : %s\n",desthost);
			printf("Sending data from: %s\n",srchost);
			printf("Random source IPs: %s\n",rand_saddr ? "Yes" : "No");
			printf("Encoded filename : %s\n",filename);
			printf("\nClient Mode      : Sending data.\n\n");
		} 
		send_packets(source_host,dest_host,filename,quiet);
	}
	else /* server mode it is */
	{    
		if(verbose)
		{
			printf("Listening for data to  : %s\n",desthost);
			printf("Listening for data from: %s\n",srchost);
			printf("Decoded filename       : %s\n",filename);
			printf("\nServer Mode            : Listening for data.\n\n");
		}
		recv_packets(source_host,dest_host,filename,quiet);
	}
	exit(0);
}
