/* 
 * util.c - utility routines for ncovert2
 *
 * $Id: util.c,v 1.1.1.1 2004/03/17 16:55:34 s-nomad Exp $
 *
 * $Log: util.c,v $
 * Revision 1.1.1.1  2004/03/17 16:55:34  s-nomad
 * Initial import for ncovert2-1.1
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "ncovert.h"
#include "rand_gen.h"
#include "sha1.h"

/*
 * true working memset as compiler could 'optimize' out the regular memset
 */
void *guaranteed_memset(void *v, int c, size_t n)
{
	volatile char *p=v;
	while (n--) *p++=c;
	return v;
}

/*
 * rotate bytes around in a long
 */
unsigned int rotate_long(unsigned int block, int len)
{
	char onebyte;
	int i;
	blocks temp;

	temp.long_chunk = block;
	for (i=0;i<len;i++)
	{
		onebyte = temp.byte_chunk[0];
		temp.byte_chunk[0] = temp.byte_chunk[1];
		temp.byte_chunk[1] = temp.byte_chunk[2];
		temp.byte_chunk[2] = temp.byte_chunk[3];
		temp.byte_chunk[3] = onebyte;
	}
	return temp.long_chunk;
}

/*
 * simple address lookup
 */
unsigned int host_convert(char *hostname)
{
	static struct in_addr ha;
	struct hostent *he;
	ha.s_addr = inet_addr(hostname);
	if(ha.s_addr == -1)
	{
		he = gethostbyname(hostname);
		if(he == NULL)
		{
			fprintf(stderr, "cannot resolve %s\n", hostname);
			exit(0);
		}
		bcopy(he->h_addr, (char *)&ha.s_addr, he->h_length);
	}
	return ha.s_addr;
}

void init_ncovert2(int server)
{
	int i = 0, j = 0;
	unsigned short short_i, short_j, temp;
	char* pass = get_password();

	// turn the password into a hash and fill out the 320 chars
	extend_mat(hash_string_with_sha1(pass),&hashPassword[0],320);

	// we no longer need the plaintext password, so erase it
	guaranteed_memset(pass,0,strlen(pass));
	free(pass);

	if(server) return;

	// in client mode we do a little more up front....
	// seed the prng
	seed_rand_num();

	// create the session key
	key.l = get_rand_num(R_UNSIGNEDLONG);

	// the ipid for the first packet will contain half the key XOR'd with
	// the first two chars of the hashed password
	ipid.c[0] = key.c[0] ^ hashPassword[0];
	ipid.c[1] = key.c[1] ^ hashPassword[1];

	// the source port for the first packet will contain the other half of
	// the key XOR'd with the second two chars of the hashed password
	s_port.c[0] = key.c[2] ^ hashPassword[2];
	s_port.c[1] = key.c[3] ^ hashPassword[3];

	// we create a hash for tracking our packets on the server end, using
	// the session key and the hashed password
	for(i = 0; i < 320; i++)
	{
		hashX[i] = hashPassword[i] ^ key.c[j];
		j++;
		if(j == 4) j = 0;
	}

	// since we are imitating a port scan, the destination ports need to
	// be randomized
	for(short_i = 0; short_i<65535; short_i++) dest_port[short_i] = short_i;
	for(short_i = 0; short_i<65535; short_i++)
	{
		temp = dest_port[short_i];
		dest_port[short_i] = dest_port[get_rand_num(R_UNSIGNEDLONG) % 65535];
		short_j = get_rand_num(R_UNSIGNEDLONG) % 65535;
		dest_port[short_i] = dest_port[short_j];
		dest_port[short_j] = temp;
	}
}

void init_server(unsigned short id)
{ 
	int i = 0, j = 0;

	ipid.w = id; 
	key.c[0] = ipid.c[0] ^ hashPassword[0];
	key.c[1] = ipid.c[1] ^ hashPassword[1];
	key.c[2] = s_port.c[0] ^ hashPassword[2];
	key.c[3] = s_port.c[1] ^ hashPassword[3];
	for(i = 0; i < 320; i++)
	{
		hashX[i] = hashPassword[i] ^ key.c[j];
		j++;
		if(j == 4) j = 0;
	}
}

void prepare_data(SI s_port_temp,blocks block_temp, int j, unsigned long tempF)
{
	s_port.c[0] = s_port_temp.c[0] ^ hashX[j];
	s_port.c[1] = s_port_temp.c[1] ^ hashX[j + 1];
	F = tempF ^ block_temp.long_chunk;
}

void determine_next_source_port(SI s_port_temp, int j)
{
	s_port.c[0] = s_port_temp.c[0] ^ hashX[j];
	s_port.c[1] = s_port_temp.c[1] ^ hashX[j + 1];
}

/*
 * handle alarm
 */
void caughtsig(int sig)
{
	if (verbose) printf("Timed out\n");
	exit(sig);
}

/* 
 * generic cksum routine is from old ping.c, note the copyright
 */
/* Copyright (c)1987 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * dupliated in all such forms and that any documentation, advertising 
 * materials, and other materials related to such distribution and use
 * acknowledge that the software was developed by the University of
 * California, Berkeley. The name of the University may not be used
 * to endorse or promote products derived from this software without
 * specific prior written permission. THIS SOFTWARE IS PROVIDED ``AS
 * IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHATIBILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE
 */
unsigned short in_cksum(unsigned short *ptr, int nbytes)
{
  register long           sum;
  u_short                 oddbyte;
  register u_short        answer;

  sum = 0;
  while (nbytes > 1) {sum += *ptr++; nbytes -= 2;}
  if (nbytes == 1) {oddbyte = 0; *((u_char *) &oddbyte) = *(u_char *)ptr; sum += oddbyte;}
  sum  = (sum >> 16) + (sum & 0xffff);
  sum += (sum >> 16);                     
  answer = ~sum;
  return(answer);
}


